"""
Video plugin
############

Plugin for Django CMS QE providing video players for hosting services like
YouTube or also custom local file. For local files is used HTML5 player.

Usage
*****

Add plugin from section **Video player** called **Video player - source file**
or **Video player - hosting services**.

Video player - source file
==========================

Choose or upload a source file at configuration screen and setup player
parameters if needed.

Track
-----

You can also add subtitles, captions, descriptions or chapters to your video.
Add sub-plugin **Track** and choose which kind of track to add.

Video player - hosting services
===============================

Choose which video hosting service you want to use at configuration screen
and setup player parameters if needed.

API
***

Django CMS
==========

.. autoclass:: cms_qe_video.cms_plugins.SourceFileVideoPlayerPlugin
    :members:

.. autoclass:: cms_qe_video.cms_plugins.HostingVideoPlayerPlugin
    :members:

.. autoclass:: cms_qe_video.cms_plugins.VideoTrackPlugin
    :members:

"""



# -*- coding: utf-8 -*-
# Generated by Django 1.10.7 on 2017-09-22 15:43
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('cms', '0016_auto_20160608_1535'),
    ]

    operations = [
        migrations.CreateModel(
            name='MailingList',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=50)),
                ('external_service', models.PositiveSmallIntegerField(choices=[(1, 'None'), (2, 'Mailchimp')], default=1)),
                ('external_id', models.CharField(blank=True, max_length=10, null=True)),
            ],
        ),
        migrations.CreateModel(
            name='NewsletterPluginModel',
            fields=[
                ('cmsplugin_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, related_name='cms_qe_newsletter_newsletterpluginmodel', serialize=False, to='cms.CMSPlugin')),
                ('title', models.CharField(blank=True, help_text='The title of subscribes that will be shown to users', max_length=150, verbose_name='Title')),
                ('fullname_show', models.BooleanField(default=False, verbose_name='Show fullname fields')),
                ('fullname_require', models.BooleanField(default=False, verbose_name='Require fullname')),
                ('mailing_lists', models.ManyToManyField(to='cms_qe_newsletter.MailingList', verbose_name='Maillist')),
            ],
            options={
                'abstract': False,
            },
            bases=('cms.cmsplugin',),
        ),
        migrations.CreateModel(
            name='Subscriber',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('email', models.EmailField(max_length=254)),
                ('first_name', models.CharField(blank=True, max_length=50)),
                ('last_name', models.CharField(blank=True, max_length=50)),
                ('external_id', models.CharField(blank=True, max_length=50, null=True)),
                ('mailing_list', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='cms_qe_newsletter.MailingList')),
            ],
        ),
        migrations.CreateModel(
            name='SubscribeTask',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('email', models.EmailField(max_length=254)),
                ('first_name', models.CharField(max_length=50, null=True)),
                ('last_name', models.CharField(max_length=50, null=True)),
                ('external_id', models.CharField(max_length=10, null=True)),
                ('type', models.PositiveSmallIntegerField(choices=[(1, 'Subscribe'), (2, 'Unsubscribe')])),
                ('created', models.DateTimeField(auto_now=True)),
                ('attempts', models.PositiveSmallIntegerField(default=0)),
                ('last_error', models.TextField()),
                ('mailing_list', models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, to='cms_qe_newsletter.MailingList')),
            ],
        ),
    ]



import re
from typing import Iterable

from cms.models import CMSPlugin
from django.conf import settings
from django.core.exceptions import ValidationError
from django.db import models
from django.utils.translation import ugettext_lazy as _
from django.utils.translation import ugettext
from djangocms_attributes_field.fields import AttributesField
from filer.fields.file import FilerFileField
from filer.fields.image import FilerImageField

# mp4, are required for full browser support
ALLOWED_EXTENSIONS = getattr(
    settings,
    'DJANGOCMS_VIDEO_ALLOWED_EXTENSIONS',
    ['mp4', 'webm', 'ogv'],
)


class AbstractVideoPlayer(CMSPlugin):
    """
    Abstract configuration model for video player.
    """
    label = models.CharField(
        verbose_name=_('Label'),
        blank=True,
        max_length=255,
    )
    poster = FilerImageField(
        verbose_name=_('Poster'),
        blank=True,
        null=True,
        on_delete=models.SET_NULL,
        related_name='+',
    )
    width = models.IntegerField(
        verbose_name=_('Width'),
        blank=True,
        null=True,
        help_text='Leave it blank to make a video player of the default width of video source'
    )
    height = models.IntegerField(
        verbose_name=_('Height'),
        blank=True,
        null=True,
        help_text='Leave it blank to make a video player of the default width of video source'
    )
    controls = models.BooleanField(
        verbose_name=_('Show controls'),
        default=True,
    )
    autoplay = models.BooleanField(
        verbose_name=_('Autoplay'),
        default=False)
    loop = models.BooleanField(
        verbose_name=_('Loop'),
        default=False)
    other_attributes = AttributesField(
        verbose_name=_('Other attributes'),
        blank=True,
    )

    class Meta:
        abstract = True

    # Add an app namespace to related_name to avoid field name clashes
    # with any other plugins that have a field with the same name as the
    # lowercase of the class name of this model.
    # https://github.com/divio/django-cms/issues/5030
    cmsplugin_ptr = models.OneToOneField(
        CMSPlugin,
        related_name='%(app_label)s_%(class)s',
        parent_link=True,
    )

    def __str__(self):
        return self.label or str(self.pk)

    def copy_relations(self, old_instance):
        # Because we have a ForeignKey, it's required to copy over
        # the reference from the instance to the new plugin.
        self.poster = old_instance.poster

    def _get_attributes_str_to_html(self, attributes: Iterable[str]) -> str:
        """
        Return string with attributes to add to HTML tag e.g.:
        width="500" autoplay mute
        """
        return ' '.join(
            ["{}".format(attribute) if isinstance(value, bool) else "{}={}".format(attribute, value)
             for attribute, value in self.__dict__.items() if attribute in attributes and value]
        )

    def _get_attributes_str_to_url(self, attributes: Iterable[str]) -> str:
        """
        Return string with attributes to add to URL e.g.:
        width=500&autoplay=0&mute=0
        """
        return '&'.join(
            ["{}={}".format(attribute, int(value)) for attribute, value in self.__dict__.items()
             if attribute in attributes and value]
        )

    @property
    def attributes_str_to_html(self) -> str:
        """
        Return height and width attributes to put them to html tag. Looks like:
        height="{value}" width="{value}" etc
        """
        attributes_to_print = ('width', 'height', 'controls', 'autoplay', 'loop')
        return self._get_attributes_str_to_html(attributes_to_print)

    @property
    def attributes_str_to_url(self) -> str:
        """
        Return height and width attributes to put them to url. Looks like:
        controls={value}&width={value} etc.
        """
        attributes_to_print = ('controls', 'autoplay', 'loop')
        return self._get_attributes_str_to_url(attributes_to_print)


class SourceFileVideoPlayer(AbstractVideoPlayer):
    """
    Configuration model for video player to play video from file on local disk.
    """
    source_file = FilerFileField(
        verbose_name=_('Source'),
        blank=False,
        null=True,
        on_delete=models.SET_NULL,
        related_name='+',
    )
    text_title = models.CharField(
        verbose_name=_('Title'),
        blank=True,
        max_length=255,
    )
    text_description = models.TextField(
        verbose_name=_('Description'),
        blank=True,
    )
    attributes = AttributesField(
        verbose_name=_('Attributes'),
        blank=True,
    )
    muted = models.BooleanField(
        verbose_name=_('Mute'),
        default=False)

    def __str__(self):
        if self.label:
            return self.source_file.label
        return str(self.pk)

    def clean(self):
        if self.source_file and self.source_file.extension not in ALLOWED_EXTENSIONS:
            raise ValidationError(
                ugettext('Incorrect file type: {extension}.').format(extension=self.source_file.extension)
            )

    def get_short_description(self):
        if self.source_file.label:
            return self.source_file.label
        return ugettext('<file is missing>')

    def copy_relations(self, old_instance):
        # Because we have a ForeignKey, it's required to copy over
        # the reference from the instance to the new plugin.
        self.source_file = old_instance.source_file

    @property
    def attributes_str_to_html(self) -> str:
        """
        Overloaded.
        Add mute attribute to base-class function.
        """
        attributes_to_print = ('muted',)
        return super().attributes_str_to_html + ' ' + self._get_attributes_str_to_html(attributes_to_print)


# Video hosts services constants
YOUTUBE = 1
VIMEO = 2
OTHERS = 3


class HostingVideoPlayer(AbstractVideoPlayer):
    """
    Configuration model for video player to play video from video hosting services.
    """
    VIDEO_HOSTING_SERVICES = (
        (YOUTUBE, 'YouTube'),
        (VIMEO, 'Vimeo'),
        (OTHERS, _('Others'))
    )
    video_hosting_service = models.IntegerField(
        verbose_name=_('Video hosting service'),
        choices=VIDEO_HOSTING_SERVICES,
        default=OTHERS,
    )

    video_url = models.URLField(
        verbose_name=_('Embed link'),
        max_length=255,
        help_text=_('Use this field to embed videos from external services '
                    'such as YouTube, Vimeo or others.'),
    )

    @property
    def size_attributes_str_to_html(self) -> str:
        """
        Return height and width attributes to put them to html tag. Looks like:
        height="{value}" width="{value}"
        """
        attributes_to_print = ('height', 'width')
        return self._get_attributes_str_to_html(attributes_to_print)

    def clean(self):
        """
        Validation URLs. Function checks if URL belongs to selected video host service.
        """
        if self.video_hosting_service == VIMEO:
            if not re.search(r'(^|[/.])vimeo.com/', self.video_url):
                raise ValidationError(_('URL does not belong to Vimeo'))
            if not self.controls:
                raise ValidationError(_('Vimeo does not support hiding controls.'))

        elif self.video_hosting_service == YOUTUBE:
            if not re.search(r'(^|[/.])youtu.be/', self.video_url) and not re.search(r'(^|[/.])youtube.com/',
                                                                                     self.video_url):
                raise ValidationError(_('URL does not belong to YouTube'))


class VideoTrack(CMSPlugin):
    """
    Renders the HTML <track> element inside <video>.
    """
    KIND_CHOICES = [
        ('subtitles', _('Subtitles')),
        ('captions', _('Captions')),
        ('descriptions', _('Descriptions')),
        ('chapters', _('Chapters')),
    ]

    kind = models.CharField(
        verbose_name=_('Kind'),
        choices=KIND_CHOICES,
        max_length=255,
    )
    src = FilerFileField(
        verbose_name=_('Source file'),
        blank=False,
        null=True,
        on_delete=models.SET_NULL,
        related_name='+',
    )
    srclang = models.CharField(
        verbose_name=_('Source language'),
        blank=True,
        max_length=255,
        help_text=_('Examples: "en" or "de" etc.'),
    )
    label = models.CharField(
        verbose_name=_('Label'),
        blank=True,
        max_length=255,
    )
    attributes = AttributesField(
        verbose_name=_('Attributes'),
        blank=True,
    )

    def __str__(self):
        label = self.kind
        if self.srclang:
            label += ' {}'.format(self.srclang)
        return label



import typing
from urllib import parse

from django import template
from django.template.defaultfilters import stringfilter

register = template.Library()


@typing.no_type_check
@register.filter(name='cms_qe_video_url_to_embed')
@stringfilter
def cms_qe_video_url_to_embed(url: str) -> str:
    """
    Django template filter to change video URL to make it embed.
    A little hack for popular video hosting services.
    """

    url = parse.urlparse(url)
    if 'vimeo.com' in url.netloc:
        return parse.urlunparse((url.scheme, 'player.vimeo.com', '/video' + url.path, [], {}, ''))
    elif 'youtube.com' in url.netloc or 'youtu.be' in url.netloc:
        video_id = parse.parse_qs(url.query)['v'][0]
        return parse.urlunparse((url.scheme, 'www.youtube.com', '/embed/' + video_id, [], {}, ''))

    return parse.urlunparse(url)



from cms_qe.utils import get_email
from cms_qe_auth.token import TokenGenerator
from cms_qe_auth.utils import pk_to_uidb64
from django.contrib.auth.models import AbstractUser, Group as DjangoGroup
from django.urls import reverse
from django.utils.translation import ugettext_lazy as _


class MissingBaseURLException(Exception):
    pass


class Group(DjangoGroup):
    pass


class User(AbstractUser):
    class Meta:
        ordering = ('username',)

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self._previous_email = self.email

    def save(self, *args, **kwargs):
        # If application does not need username, it just use email instead.
        # Note that Django normalizes username so to make it work we need to change
        # behaviour of normalize_username.
        if not self.username or self.username == self._previous_email:
            self.username = self.email
        if self._previous_email and self.email != self._previous_email:
            self.is_active = False

        is_new = not self.pk

        base_url = kwargs.pop('base_url', None)
        super(User, self).save(*args, **kwargs)

        if is_new and not self.is_active:
            if not base_url:
                raise MissingBaseURLException('New user can not be saved without a base url')
            self.send_activation_email(base_url)

    @classmethod
    def normalize_username(cls, username):
        # By default Django normalizes username which converts None into "None".
        if username is None:
            return username
        return super().normalize_username(username)

    def activate(self, token):
        if not self._check_activation_token(token):
            return False
        self.is_active = True
        self.save()
        return True

    def send_activation_email(self, base_url):
        token = self._generate_activation_token()
        activation_url = self._get_absolute_activation_url(base_url=base_url, token=token)
        email = get_email(
            template='cms_qe/auth/email/activation',
            subject=_('Activate your account'),
            to=self.email,
            username=self.username,
            activation_url=activation_url,
        )
        email.send()

    def _generate_activation_token(self):
        return TokenGenerator().make_token(self)

    def _check_activation_token(self, token):
        return TokenGenerator().check_token(self, token)

    def _get_absolute_activation_url(self, base_url, token: str) -> str:
        relative_url = reverse('activate', kwargs={'uidb64': pk_to_uidb64(self.pk), 'token': token})
        return base_url + relative_url



from typing import Type, TypeVar

from cms_qe.utils import get_base_url
from cms_qe_auth.utils import get_user_by_uidb64
from django.contrib import messages
from django.contrib.auth import login
from django.http import HttpRequest
from django.shortcuts import redirect
from django.template.response import TemplateResponse
from django.utils.translation import ugettext as _

from .forms import RegisterForm

RF = TypeVar('RF', bound=RegisterForm)


def register(
        request: HttpRequest,
        template_name: str = 'cms_qe/auth/register.html',
        register_form: Type[RF] = RegisterForm,
):
    """
    Displays the register form and handles the register action.
    """
    if request.method == 'POST':
        form = register_form(data=request.POST)
        if form.is_valid():
            base_url = get_base_url(request)
            user = form.save(commit=False)
            user.is_active = False
            user.save(base_url=base_url)
            messages.success(request, _('You were successfully registered. Please confirm your email address.'))
            return redirect('register')
        else:
            messages.error(request, _('Please correct errors below.'))
    else:
        form = register_form()

    context = {'form': form}
    return TemplateResponse(request, template_name, context)


def activate(request: HttpRequest,
             uidb64: str,
             token: str,
             template_name_complete: str = 'cms_qe/auth/email_confirmation_complete.html',
             template_name_fail: str = 'cms_qe/auth/email_confirmation_fail.html',
            ):
    user = get_user_by_uidb64(uidb64)
    if user and user.activate(token):
        login(request, user)
        return TemplateResponse(request, template_name_complete)
    return TemplateResponse(request, template_name_fail)



from typing import Any, Dict, List, Tuple

from django.apps import apps
from django.db import models
from django.db.models import Q
from django.db.models.base import ModelBase
from django.contrib.auth import get_user_model

from .exceptions import TableDoesNotExists


SUPPORTED_FIELDS = (
    models.AutoField,
    models.BigAutoField,
    models.BigIntegerField,
    models.BooleanField,
    models.CharField,
    models.DateField,
    models.DateTimeField,
    models.DecimalField,
    models.EmailField,
    models.FloatField,
    models.ForeignKey,
    models.IntegerField,
    models.GenericIPAddressField,
    models.NullBooleanField,
    models.PositiveIntegerField,
    models.PositiveSmallIntegerField,
    models.SlugField,
    models.SmallIntegerField,
    models.TextField,
    models.TimeField,
    models.URLField,
    models.UUIDField,
)


def get_model_by_table(table: str) -> ModelBase:
    """
    Returns Django model by table name.
    """
    if table == 'auth_user':
        return get_user_model()

    for app_models in apps.all_models.values():
        for cls in app_models.values():
            if cls._meta.db_table == table:
                return cls
    raise TableDoesNotExists(table)


def get_models_choices() -> Tuple[Tuple[str, Tuple[Tuple[str, str], ...]], ...]:
    """
    Returns sorted and grouped choices of all models per app.

    Example output:

    .. code-block:: txt

        (
            'app1',
            (
                ('choice1 key', 'choice1 name'),
                ...
            ),
        ), (
            'app2',
            (
                ('choice2 key', 'choice2 name'),
                ...
            ),
        ), (
            ...
        )
    """
    return tuple((
        app,
        tuple(
            (cls._meta.db_table, cls._meta.object_name)
            for model, cls in sorted(app_models.items())
        ),
    ) for app, app_models in sorted(apps.all_models.items()) if app_models)


def get_table_choices(table: str) -> Dict[str, List[Tuple[str, str, str]]]:
    """
    Returns choices for table depending on exact table name.

    .. code-block:: txt

        {
            "columns": [
                ["name", "label", "type"],
                ...
            ]
        }
    """
    model = get_model_by_table(table)
    choices = {
        'columns': [
            (
                field.name,
                getattr(field, 'verbose_name', field.name),
                get_field_type(field),
            )
            for field in model._meta.get_fields()
            if field.__class__ in SUPPORTED_FIELDS
        ],
    }
    return choices


def get_field_type(field) -> str:
    if isinstance(field, (models.BooleanField, models.NullBooleanField)):
        return 'boolean'
    if isinstance(field, (models.AutoField, models.IntegerField)):
        return 'integer'
    if isinstance(field, (models.DecimalField, models.FloatField)):
        return 'float'
    return 'string'


def get_filter_params(model, filter_data: Dict[str, Any]) -> Tuple[List[Any], Dict[str, Any]]:
    """
    Returns tuple with args and kwargs for queryset filter.
    """
    filter_args = []
    filter_kwds = {}

    # Let's add only fields which are in the model.
    # When some field is missing (for example after upgrade),
    # we still want to be able to not end up with internal error.
    for field in model._meta.get_fields():
        if field.name not in filter_data:
            continue

        key = field.name
        value = filter_data[field.name]

        # This presumes that ForeignKey is actually integer. As user want to filter
        # by ID, then nothing extra needs to happen, but for string we simply find
        # all strings in related table and check if that string is present.
        if isinstance(field, models.ForeignKey) and isinstance(value, str):
            foreign_filter = Q()
            for foreign_field in field.rel.model._meta.get_fields():
                if isinstance(foreign_field, (models.CharField, models.TextField)):
                    foreign_filter |= Q(**{'{}__{}__icontains'.format(key, foreign_field.name): value})
            filter_args.append(foreign_filter)

        # Any text field search as LIKE, not exact match.
        elif isinstance(field, (models.CharField, models.TextField)):
            filter_kwds['{}__icontains'.format(key)] = value

        else:
            filter_kwds[key] = value

    return filter_args, filter_kwds



"""
Newsletter plugin
#################

Plugin for Django CMS QE providing newsletter. It allows users to subscribe
(simply add email to database) and also possibility to sync with external
sources like MailChimp.

Usage
*****

Add plugin from section **Generic** called **Newsletter subscription form**.

To synchronize emails with MailChimp, `generate a api_key on MailChimp
<http://kb.mailchimp.com/integrations/api-integrations/about-api-keys>`_
and set it with MailChimp username (which you can find or set in `profile settings
<https://admin.mailchimp.com/account/profile/>`_) in admin interface in *Сonstance*.
Then you can synchronize mailing lists in list of mailing lists in admin interface
and use those lists in your plugin.

API
***

Django CMS
==========

.. autoclass:: cms_qe_newsletter.cms_plugins.NewsletterPlugin
    :members:

.. autoclass:: cms_qe_newsletter.models.NewsletterPluginModel
    :members:

Models
======

.. autoclass:: cms_qe_newsletter.models.MailingList
    :members:

.. autoclass:: cms_qe_newsletter.models.Subscriber
    :members:

.. autoclass:: cms_qe_newsletter.models.SubscribeTask
    :members:

Views
=====

.. automodule:: cms_qe_newsletter.views
    :members:

Manage
======

.. autoclass:: cms_qe_newsletter.management.commands.cms_qe_newsletter_sync.Command
    :members:

Services
========

.. automodule:: cms_qe_newsletter.external_services.sync
    :members:

Mailchimp
---------

.. automodule:: cms_qe_newsletter.external_services.mailchimp.sync
    :members:

.. automodule:: cms_qe_newsletter.external_services.mailchimp.client
    :members:
"""

import logging
logger = logging.getLogger(__name__)

default_app_config = 'cms_qe_newsletter.apps.NewsletterConfig'



from unittest.mock import patch

import pytest
from cms_qe.utils import get_base_url, get_functions, get_module, get_modules
from django.test import override_settings


@pytest.mark.parametrize('expect,module_name', [
    (True, 'views'),
    (True, 'views.monitoring'),
    (False, 'no_module'),
    (False, 'monitoring.get_status'),
])
def test_get_module(expect, module_name):
    module = get_module('cms_qe', module_name)
    assert module if expect else module is None


def test_get_modules():
    modules = get_modules('monitoring')
    assert [module for app_name, module in modules if app_name == 'cms_qe']


def test_get_functions():
    functions = get_functions('monitoring', 'get_status')
    assert [function for app_name, function in functions if app_name == 'cms_qe']


def test_get_functions_existing_module_without_function():
    functions = get_functions('monitoring', 'no_function')
    assert not list(functions)


@override_settings(META_SITE_PROTOCOL='https', )
def test_get_absolute_url(post_request_empty):
    request = post_request_empty
    mock_url = 'test.com'
    with patch('cms_qe.utils.get_current_site', return_value=mock_url):
        base_url = get_base_url(request)
    assert base_url == 'https://{}'.format(mock_url)



"""
Tests for the analytical.utils module.
"""
# import django

from django.contrib.auth.models import AbstractBaseUser
from django.db import models
from django.http import HttpRequest
from django.template import Context
from django.test.utils import override_settings

from ..utils import (
    AnalyticalException,
    get_domain,
    get_identity,
    get_required_setting,
    is_internal_ip,
)
from ..tests.utils import TestCase


class SettingDeletedTestCase(TestCase):

    @override_settings(USER_ID=None)
    def test_get_required_setting(self):
        """
        Make sure using get_required_setting fails in the right place.
        """

        # available in python >= 3.2
        if hasattr(self, 'assertRaisesRegex'):
            with self.assertRaisesRegex(AnalyticalException, "USER_ID"):
                user_id = get_required_setting("USER_ID", "\d+", "invalid USER_ID")
        else:
            self.assertRaises(AnalyticalException,
                              get_required_setting, "USER_ID", "\d+", "invalid USER_ID")


class MyUser(AbstractBaseUser):
    identity = models.CharField(max_length=50)
    USERNAME_FIELD = 'identity'


class GetIdentityTestCase(TestCase):
    def test_custom_username_field(self):
        get_id = get_identity(Context({}), user=MyUser(identity='fake_id'))
        self.assertEqual(get_id, 'fake_id')


@override_settings(ANALYTICAL_DOMAIN="example.org")
class GetDomainTestCase(TestCase):
    def test_get_service_domain_from_context(self):
        context = Context({'test_domain': 'example.com'})
        self.assertEqual(get_domain(context, 'test'), 'example.com')

    def test_get_analytical_domain_from_context(self):
        context = Context({'analytical_domain': 'example.com'})
        self.assertEqual(get_domain(context, 'test'), 'example.com')

    @override_settings(TEST_DOMAIN="example.net")
    def test_get_service_domain_from_settings(self):
        context = Context()
        self.assertEqual(get_domain(context, 'test'), 'example.net')

    def test_get_analytical_domain_from_settings(self):
        context = Context()
        self.assertEqual(get_domain(context, 'test'), 'example.org')


class InternalIpTestCase(TestCase):

    @override_settings(ANALYTICAL_INTERNAL_IPS=['1.1.1.1'])
    def test_render_no_internal_ip(self):
        context = Context()
        self.assertFalse(is_internal_ip(context))

    @override_settings(INTERNAL_IPS=['1.1.1.1'])
    @override_settings(ANALYTICAL_INTERNAL_IPS=[])
    def test_render_analytical_internal_ips_override_when_empty(self):
        req = HttpRequest()
        req.META['REMOTE_ADDR'] = '1.1.1.1'
        context = Context({'request': req})
        self.assertFalse(is_internal_ip(context))

    @override_settings(ANALYTICAL_INTERNAL_IPS=['1.1.1.1'])
    def test_render_internal_ip(self):
        req = HttpRequest()
        req.META['REMOTE_ADDR'] = '1.1.1.1'
        context = Context({'request': req})
        self.assertTrue(is_internal_ip(context))

    @override_settings(TEST_INTERNAL_IPS=['1.1.1.1'])
    def test_render_prefix_internal_ip(self):
        req = HttpRequest()
        req.META['REMOTE_ADDR'] = '1.1.1.1'
        context = Context({'request': req})
        self.assertTrue(is_internal_ip(context, 'TEST'))

    @override_settings(INTERNAL_IPS=['1.1.1.1'])
    def test_render_internal_ip_fallback(self):
        req = HttpRequest()
        req.META['REMOTE_ADDR'] = '1.1.1.1'
        context = Context({'request': req})
        self.assertTrue(is_internal_ip(context))

    @override_settings(ANALYTICAL_INTERNAL_IPS=['1.1.1.1'])
    def test_render_internal_ip_forwarded_for(self):
        req = HttpRequest()
        req.META['HTTP_X_FORWARDED_FOR'] = '1.1.1.1'
        context = Context({'request': req})
        self.assertTrue(is_internal_ip(context))

    @override_settings(ANALYTICAL_INTERNAL_IPS=['1.1.1.1'])
    def test_render_different_internal_ip(self):
        req = HttpRequest()
        req.META['REMOTE_ADDR'] = '2.2.2.2'
        context = Context({'request': req})
        self.assertFalse(is_internal_ip(context))



# -*- coding: utf-8 -*-
# Generated by Django 1.10.7 on 2017-09-20 09:54
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion
import djangocms_attributes_field.fields
import filer.fields.file
import filer.fields.image


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('cms', '0016_auto_20160608_1535'),
        ('filer', '0007_auto_20161016_1055'),
    ]

    operations = [
        migrations.CreateModel(
            name='HostingVideoPlayer',
            fields=[
                ('label', models.CharField(blank=True, max_length=255, verbose_name='Label')),
                ('width', models.IntegerField(blank=True, help_text='Leave it blank to make a video player of the default width of video source', null=True, verbose_name='Width')),
                ('height', models.IntegerField(blank=True, help_text='Leave it blank to make a video player of the default width of video source', null=True, verbose_name='Height')),
                ('controls', models.BooleanField(default=True, verbose_name='Show controls')),
                ('autoplay', models.BooleanField(default=False, verbose_name='Autoplay')),
                ('loop', models.BooleanField(default=False, verbose_name='Loop')),
                ('other_attributes', djangocms_attributes_field.fields.AttributesField(blank=True, default=dict, verbose_name='Other attributes')),
                ('cmsplugin_ptr', models.OneToOneField(on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, related_name='cms_qe_video_hostingvideoplayer', serialize=False, to='cms.CMSPlugin')),
                ('video_hosting_service', models.IntegerField(choices=[(1, 'YouTube'), (2, 'Vimeo'), (3, 'Others')], default=3, verbose_name='Video hosting service')),
                ('video_url', models.URLField(help_text='Use this field to embed videos from external services such as YouTube, Vimeo or others.', max_length=255, verbose_name='Embed link')),
                ('poster', filer.fields.image.FilerImageField(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='+', to='filer.Image', verbose_name='Poster')),
            ],
            options={
                'abstract': False,
            },
            bases=('cms.cmsplugin',),
        ),
        migrations.CreateModel(
            name='SourceFileVideoPlayer',
            fields=[
                ('label', models.CharField(blank=True, max_length=255, verbose_name='Label')),
                ('width', models.IntegerField(blank=True, help_text='Leave it blank to make a video player of the default width of video source', null=True, verbose_name='Width')),
                ('height', models.IntegerField(blank=True, help_text='Leave it blank to make a video player of the default width of video source', null=True, verbose_name='Height')),
                ('controls', models.BooleanField(default=True, verbose_name='Show controls')),
                ('autoplay', models.BooleanField(default=False, verbose_name='Autoplay')),
                ('loop', models.BooleanField(default=False, verbose_name='Loop')),
                ('other_attributes', djangocms_attributes_field.fields.AttributesField(blank=True, default=dict, verbose_name='Other attributes')),
                ('cmsplugin_ptr', models.OneToOneField(on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, related_name='cms_qe_video_sourcefilevideoplayer', serialize=False, to='cms.CMSPlugin')),
                ('text_title', models.CharField(blank=True, max_length=255, verbose_name='Title')),
                ('text_description', models.TextField(blank=True, verbose_name='Description')),
                ('attributes', djangocms_attributes_field.fields.AttributesField(blank=True, default=dict, verbose_name='Attributes')),
                ('muted', models.BooleanField(default=False, verbose_name='Mute')),
                ('poster', filer.fields.image.FilerImageField(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='+', to='filer.Image', verbose_name='Poster')),
                ('source_file', filer.fields.file.FilerFileField(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='+', to='filer.File', verbose_name='Source')),
            ],
            options={
                'abstract': False,
            },
            bases=('cms.cmsplugin',),
        ),
        migrations.CreateModel(
            name='VideoTrack',
            fields=[
                ('cmsplugin_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, related_name='cms_qe_video_videotrack', serialize=False, to='cms.CMSPlugin')),
                ('kind', models.CharField(choices=[('subtitles', 'Subtitles'), ('captions', 'Captions'), ('descriptions', 'Descriptions'), ('chapters', 'Chapters')], max_length=255, verbose_name='Kind')),
                ('srclang', models.CharField(blank=True, help_text='Examples: "en" or "de" etc.', max_length=255, verbose_name='Source language')),
                ('label', models.CharField(blank=True, max_length=255, verbose_name='Label')),
                ('attributes', djangocms_attributes_field.fields.AttributesField(blank=True, default=dict, verbose_name='Attributes')),
                ('src', filer.fields.file.FilerFileField(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='+', to='filer.File', verbose_name='Source file')),
            ],
            options={
                'abstract': False,
            },
            bases=('cms.cmsplugin',),
        ),
    ]



"""
Menu plugin
###########

Plugin for Django CMS QE providing menu. It allows include menu at any
place so it can be moved without support of developers.

Usage
*****

Add plugin from section **Generic** called **Menu**. When used, you
are asked which page you want to use as top of the menu tree. That
is set by page ID in advanced setting. Also how deep it menu should
be. Keep in mind that normally menu supports only one level because
on mobile devices are deeper menus harder to use. Use deeper level
only when your design supports it.

API
***

.. autoclass:: cms_qe_menu.cms_plugins.MenuPlugin
    :members:

.. autoclass:: cms_qe_menu.models.MenuPluginModel
    :members:

"""



"""
Caching setting by default in-memory without need to configure anything.
"""

# Caching
# https://docs.djangoproject.com/en/1.11/topics/cache/

CACHES = {
    'default': {
        'BACKEND': 'django.core.cache.backends.memcached.MemcachedCache',
        'LOCATION': '127.0.0.1:11211',
    }
}



from time import time
from unittest import mock

from constance.test import override_config
from django.conf import settings
import pytest
from pytest_data import get_data

from .models import (
    SUBSCRIBE,
    SERVICE_MAILCHIMP,
    NewsletterPluginModel,
    MailingList,
    Subscriber,
    SubscribeTask,
)


@pytest.fixture
def cms_qe_newsletter_plugin(request):
    return NewsletterPluginModel(**get_data(
        request,
        'cms_qe_newsletter_plugin_data',
        {
            'title': 'Test Title',
            'fullname_show': False,
            'fullname_require': False,
        }
    ))


@pytest.fixture
def cms_qe_mailing_list(request):
    mailing_list = MailingList(**get_data(
        request,
        'cms_qe_mailing_list_data',
        {
            'name': 'Mail list',
            'external_service': SERVICE_MAILCHIMP,
            'external_id': settings.TEST_MAILCHIMP_LIST_ID,
        }
    ))
    mailing_list.save()
    return mailing_list


@pytest.fixture
def cms_qe_subscriber(request, cms_qe_mailing_list):
    subscriber = Subscriber(**get_data(
        request,
        'cms_qe_subscriber_data',
        {
            'mailing_list': cms_qe_mailing_list,
            'email': 'test{}@example.com'.format(time()),
            'first_name': 'Firstname',
            'last_name': 'Lastname',
        }
    ))
    subscriber.save()
    return subscriber


@pytest.fixture
def cms_qe_subscribe_task(request, cms_qe_mailing_list):
    subscribe_task = SubscribeTask(**get_data(
        request,
        'cms_qe_subscribe_task_data',
        {
            'mailing_list': cms_qe_mailing_list,
            'email': 'test{}@example.com'.format(time()),
            'first_name': 'Firstname',
            'last_name': 'Lastname',
            'type': SUBSCRIBE,
        }
    ))
    subscribe_task.save()
    return subscribe_task


@pytest.fixture
def mock_mailchimp(request):
    mailchimp_mock = mock.Mock()
    mailchimp_mock.lists.all.return_value = get_data(request, 'cms_qe_mailchimp_list_data', {
        'lists': [
            {'name': 'some_name_1', 'id': 'some_id_1'},
            {'name': 'some_name_2', 'id': 'some_id_2'},
        ],
    })
    mailchimp_mock.lists.members.create.return_value = get_data(request, 'cms_qe_mailchimp_subscribe_data', {
        'id': 'some_test_id',
    })
    mailchimp_mock.lists.members.delete.return_value = None
    with mock.patch('mailchimp3.MailChimp', return_value=mailchimp_mock):
        constance = override_config(
            MAILCHIMP_USERNAME=settings.TEST_MAILCHIMP_USERNAME,
            MAILCHIMP_API_KEY=settings.TEST_MAILCHIMP_API_KEY,
        )
        constance.enable()
        yield
        constance.disable()



"""
Paths settings for Django app.
"""

# Static files (CSS, JavaScript, Images)
# https://docs.djangoproject.com/en/1.11/howto/static-files/

STATICFILES_STORAGE = 'cms_qe.staticfiles.ManifestStaticFilesStorage'

STATICFILES_FINDERS = (
    'django.contrib.staticfiles.finders.FileSystemFinder',
    'aldryn_boilerplates.staticfile_finders.AppDirectoriesFinder',  # Loader of Aldryn Boilerplate static files.
    'django.contrib.staticfiles.finders.AppDirectoriesFinder',
)

STATIC_URL = '/static/'

MEDIA_URL = '/media/'



from operator import itemgetter
import re

from cms.plugin_base import CMSPluginBase
from cms.plugin_pool import plugin_pool
from django.conf import settings
from django.utils.translation import ugettext_lazy as _, get_language


@plugin_pool.register_plugin
class LanguageSwitcherPlugin(CMSPluginBase):
    """
    CMS plugin allowing to add dynamically configured listing of any model.
    """

    name = _('Language switcher')
    render_template = 'cms_qe/i18n/language_switcher.html'
    text_enabled = True
    cache = True

    def render(self, context: dict, instance, placeholder) -> dict:
        context = super().render(context, instance, placeholder)
        context.update({
            'path': self.path_without_language(context),
            'language': get_language(),
            'languages': self.sorted_languages,
        })
        return context

    def path_without_language(self, context):
        return re.sub(r'^/[a-z]{2}/', '/', context['request'].path)

    @property
    def sorted_languages(self):
        return sorted(settings.LANGUAGES, key=itemgetter(0))



from django.utils.translation import ugettext_lazy as _

from .export import register_export_action


register_export_action('csv', _('Export selected as CSV'))
register_export_action('xls', _('Export selected as XLS'))
register_export_action('json', _('Export selected as JSON'))
register_export_action('html', _('Export selected as HTML'))



"""
Base configuration for quick and easy production use.

By default enables all security options, all useful plugins, etc.
"""

# pylint: disable=wildcard-import
from .app import *
from .auth import *
from .cache import *
from .cms import *
from .constants import *
from .database import *
from .email import *
from .i18n import *
from .logging import *
from .path import *
from .security import *
from .template import *



"""
django-analytical testing settings.
"""

DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.sqlite3',
        'NAME': ':memory:',
    }
}

INSTALLED_APPS = [
    'django.contrib.sites',
    'django.contrib.contenttypes',
    'django.contrib.auth',
    'analytical',
]

SECRET_KEY = 'testing'

MIDDLEWARE_CLASSES = (
    'django.middleware.common.CommonMiddleware',
    'django.middleware.csrf.CsrfViewMiddleware',
)



"""
Base settings for Django app.
"""

SITE_ID = 1

INTERNAL_IPS = ['127.0.0.1']

META_USE_SITES = True
META_SITE_PROTOCOL = 'https'

INSTALLED_APPS = [
    # This app. :-)
    # It is holding at the top of the list, so that allow rewrite the templates in third side applications.
    'cms_qe',
    'cms_qe_auth',
    'cms_qe_breadcrumb',
    'cms_qe_i18n',
    'cms_qe_menu',
    'cms_qe_newsletter',
    'cms_qe_table',
    'cms_qe_video',
    'cms_qe_analytical',

    # Must be before django.contrib.admin.
    'djangocms_admin_style',

    # Django's defaults.
    'django.contrib.admin',
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'django.contrib.messages',
    'django.contrib.staticfiles',
    'django.contrib.sites',  # Needed by Django CMS.
    'django.contrib.sitemaps',

    # Django CMS's core modules.
    'cms',
    'menus',
    'treebeard',  # Tree structure of pages and plugins.
    'sekizai',  # Static file management.

    # Other Django CMS's useful modules.
    'djangocms_text_ckeditor',
    'djangocms_googlemap',
    'aldryn_bootstrap3',
    'aldryn_boilerplates',
    'aldryn_forms',
    'aldryn_forms.contrib.email_notifications',
    'captcha',  # Needed by Aldryn Forms.
    'djangocms_inline_comment',

    # Django Filer's modules.
    'filer',
    'easy_thumbnails',
    'mptt',

    # Other Django Files's useful modules.
    'cmsplugin_filer_file',
    'cmsplugin_filer_folder',
    'cmsplugin_filer_link',
    'cmsplugin_filer_image',

    # Other Django's modules.
    'axes',
    'bootstrapform',
    'constance',
    'constance.backends.database',
    'import_export',
]

MIDDLEWARE = [
    # Must be the first. Cache is more important, second one is only
    # for development auto-reload also after apphook changes.
    'django.middleware.cache.UpdateCacheMiddleware',
    'cms.middleware.utils.ApphookReloadMiddleware',

    # Django's defaults.
    'django.middleware.security.SecurityMiddleware',
    'django.contrib.sessions.middleware.SessionMiddleware',
    'django.middleware.locale.LocaleMiddleware',  # Locale is mandatory by Django CMS.
    'django.middleware.common.CommonMiddleware',
    'django.middleware.csrf.CsrfViewMiddleware',
    'django.contrib.auth.middleware.AuthenticationMiddleware',
    'django.contrib.messages.middleware.MessageMiddleware',
    'django.middleware.clickjacking.XFrameOptionsMiddleware',

    # Extra Django's middlewares.
    'django.middleware.common.BrokenLinkEmailsMiddleware',

    # Django CMS's core middlewares.
    'cms.middleware.user.CurrentUserMiddleware',
    'cms.middleware.page.CurrentPageMiddleware',
    'cms.middleware.toolbar.ToolbarMiddleware',
    'cms.middleware.language.LanguageCookieMiddleware',

    # Security middleware.
    'csp.middleware.CSPMiddleware',

    # Must be the last.
    'django.middleware.cache.FetchFromCacheMiddleware',
]



"""
URL Configuration
https://docs.djangoproject.com/en/1.11/topics/http/urls/
"""

from django.conf.urls import url

from .views import update_lists


urlpatterns = [
    url(r'^cms-qe/newsletter/sync-lists', update_lists, name='sync_mailing_lists'),
]



from django.http import JsonResponse

from cms_qe.utils import get_functions


def get_monitoring(request):  # pylint: disable=unused-argument
    """
    Returns JSON response with data about monitoring of application.
    """
    result = get_monitoring_data()
    status_code = 200 if result['status'] else 500
    return JsonResponse(result, status=status_code)


def get_monitoring_data():
    """
    Helper to check all installed apps. It looks for function ``get_status`` in
    modules ``monitoring``. If you want to add check for your module, create your
    file called ``monitoring.py`` and add something like this:

    .. code-block:: python

        def get_status():
            # check
            return True  # or False or error message or dict

    Result is dictionary of two keys, overall ``status`` which is ``True`` is all
    calls returns ``True``, in other cases ``False``. Second key is ``app_details``
    with dictionary with key of every app which has monitoring and it's status.
    Status can be bool or any message which means something is wrong.

    You can also return dictionary with the details of your app. For example::

        {
            'databse': True,
            'cache': 'problem to connect',
            # ...
        }
    """
    result = {
        'status': True,
        'app_details': {},
    }
    for app_name, monitoring_function in get_functions('monitoring', 'get_status'):
        try:
            app_state = monitoring_function()
        except Exception as exc:  # pylint: disable=broad-except
            app_state = str(exc)
            is_ok = False
        else:
            if isinstance(app_state, dict):
                is_ok = all(v is True for v in app_state.values())
            else:
                is_ok = app_state is True

        result['app_details'][app_name] = app_state
        result['status'] &= is_ok
    return result




import os

from cms_qe.settings.dev import *


INSTALLED_APPS += [
    'example',
]

ROOT_URLCONF = 'example.urls'
WSGI_APPLICATION = 'example.wsgi.application'

BASE_DIR = os.path.join(os.path.dirname(os.path.abspath(__file__)), '..')
STATIC_ROOT = os.path.join(BASE_DIR, 'static')
MEDIA_ROOT = os.path.join(BASE_DIR, 'media')

DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.sqlite3',
        'NAME': os.path.join(BASE_DIR, '..', 'db.sqlite3'),
        'TEST': {
            'NAME': ':memory:',
        }
    }
}




class LoginPage:
    def __init__(self, driver):
        self._driver = driver

    def open(self):
        self._driver.go_to('/admin')
        return self

    def login(self, username, password=None):
        """
        Log in to Django CMS application as `username` with `password`.
        If `password` is empty, then is used same as `username`.
        """
        self._driver.get_elm('login-form').fill_out_and_submit({
            'username': username,
            'password': password or username,
        })



"""
Language plugin
###############

Plugin for Django CMS QE providing language switcher. It allows to add
links for changing language of current page by changing URL.

Usage
*****

Add plugin from section **Generic** called **Language Switcher**.

API
***

.. autoclass:: cms_qe_i18n.cms_plugins.LanguageSwitcherPlugin
    :members:

"""



"""
URL Configuration
https://docs.djangoproject.com/en/1.11/topics/http/urls/
"""

from cms.sitemaps import CMSSitemap
from django.conf import settings
from django.conf.urls import include, url
from django.conf.urls.i18n import i18n_patterns
from django.conf.urls.static import static
from django.contrib import admin
from django.contrib.sitemaps.views import sitemap
from django.views.i18n import JavaScriptCatalog

from . import views

__all__ = (
    'handler403',
    'handler404',
    'handler500',
    'handler503',
    'urlpatterns',
)

# pylint: disable=invalid-name
handler403 = 'cms_qe.views.handler403'
handler404 = 'cms_qe.views.handler404'
handler500 = 'cms_qe.views.handler500'
handler503 = 'cms_qe.views.handler503'

urlpatterns = [
    url(r'^jsi18n/$', JavaScriptCatalog.as_view(), name='javascript-catalog'),
    url(r'^', include('filer.server.urls')),
    url(r'^csp-report', views.csp_report),
    url(r'^', include('cms_qe_table.urls')),
    url(r'^', include('cms_qe_newsletter.urls')),
    url(r'^captcha/', include('captcha.urls')),
    url(r'^sitemap\.xml$', sitemap, {'sitemaps': {'cmspages': CMSSitemap}}),
    url(r'api/monitoring', views.get_monitoring),
]


# During development is error page replaced by Django error page with debug info.
# This is registration special URLs for testing error pages in dev mode.
if settings.DEBUG:
    import debug_toolbar

    urlpatterns += static(settings.MEDIA_URL, document_root=settings.MEDIA_ROOT)
    urlpatterns += [
        url(r'^__debug__/', include(debug_toolbar.urls)),
        url(r'^403/$', views.handler403),
        url(r'^404/$', views.handler404),
        url(r'^503/$', views.handler503),
    ]

# Django CMS has to be the last one because it will consume all URLs.
urlpatterns += i18n_patterns(
    url(r'^admin/', include(admin.site.urls)),
    url(r'^auth/', include('cms_qe_auth.urls')),
    url(r'^', include('cms.urls')),
)



"""
Logging settings with base formatters and handlers.
"""

# Logging
# https://docs.djangoproject.com/en/1.11/topics/logging/

SERVER_EMAIL = 'django_cms_qe@localhost'

ADMINS = (
    ('Michal Hořejšek', 'michal.horejsek@nic.cz'),
)
MANAGERS = ADMINS


LOGGING = {
    'version': 1,
    'disable_existing_loggers': False,
    'formatters': {
        'short': {
            'format': '%(levelname)s [%(filename)s:%(lineno)s] %(message)s',
            'datefmt': '%d/%b/%Y %H:%M:%S'
        },
        'long': {
            'format': '[%(asctime)s] %(levelname)s [%(pathname)s:%(lineno)s] %(message)s',
            'datefmt': '%d/%b/%Y %H:%M:%S'
        },
    },
    'handlers': {
        'console': {
            'level': 'DEBUG',
            'class': 'logging.StreamHandler',
            'formatter': 'short'
        },
        'logfile': {
            'level': 'INFO',
            'class': 'logging.handlers.RotatingFileHandler',
            'filename': '/tmp/django_cms_qe.log',
            'maxBytes': 5000000,  # 5M
            'backupCount': 0,
            'formatter': 'long',
        },
        'mailadmins': {
            'level': 'WARNING',
            'class': 'django.utils.log.AdminEmailHandler'
        },
    },
    'loggers': {
        'django': {
            'level': 'ERROR',
            'handlers': ['mailadmins'],
            'propagate': True,
        },
        'cms_qe*': {
            'level': 'WARNING',
            'handlers': ['mailadmins'],
            'propagate': True,
        },
        '': {
            'level': 'INFO',
            'handlers': ['console'],
        },
    }
}



from cms_qe_test import render_plugin
from ..cms_plugins import TablePlugin


def test_render():
    html = render_plugin(TablePlugin, table='auth_user')
    assert '<table>' in html


def test_render_not_existing_table():
    html = render_plugin(TablePlugin, table='table_does_not_exist')
    assert 'Table table_does_not_exist does not exist!' in html


def test_render_table_with_not_existing_column():
    html = render_plugin(TablePlugin, table='auth_user', columns=['column_does_not_exist'])
    assert 'Some column does not exist!' in html



import re

from cms_qe_test import render_plugin

from ..cms_plugins import HostingVideoPlayerPlugin, SourceFileVideoPlayerPlugin


def test_render_source_file_video_plugin(cms_qe_video_source_file_video_player_model):
    html = render_plugin(SourceFileVideoPlayerPlugin, cms_qe_video_source_file_video_player_model)
    assert re.search(r'<video(\s|.)*</video>', html)
    assert not re.search(r'<iframe(\s|.)*</iframe>', html)


def test_render_hosting_video_plugin(cms_qe_video_hosting_video_player_model):
    html = render_plugin(HostingVideoPlayerPlugin, cms_qe_video_hosting_video_player_model)
    assert not re.search(r'<video(\s|.)*</video>', html)
    assert re.search(r'<iframe(\s|.)*</iframe>', html)



# pylint: disable=protected-access,unused-variable,invalid-name,bad-continuation,fixme
#
# https://gist.github.com/furins/727d35367a93fd1564df
#
# templatetags/kwacros.py - Support for macros in Django templates
#
# Based on snippet by
# Author: Michal Ludvig <michal@logix.cz>
#         http://www.logix.cz/michal
#
# modified for args and kwargs by Skylar Saveland http://skyl.org
# ported to Python3 by Stefano Furin http://www.dolomitiproject.it
#

"""
Usage example:
0) Save this file as <yourapp>/templatetags/kwacros.py
1) In your template load the library:
    {% load kwacros %}
2) Define a new macro called 'my_macro' that takes two args and a kwarg.
   All will be optional.
    {% kwacro test2args1kwarg arg1 arg2 baz="Default baz" %}
    {% firstof arg1 "default arg1" %}
    {% if arg2 %}{{ arg2 }}{% else %}default arg2{% endif %}
    {{ baz }}
    {% endkwacro %}
3) Use the macro with a string parameters or context variables::
    {% usekwacro test2args1kwarg "foo" "bar" baz="KW" %}
    <br>
    {% usekwacro test2args1kwarg num_pages "bar" %}
    <br>
    {% usekwacro test2args1kwarg %}
    <br>
    {% usekwacro test2args1kwarg "new" "foobar"|join:"," baz="diff kwarg" %}
    renders like
    foo bar KW
    77 bar Default baz
    default arg1 default arg2 Default baz
    new f,o,o,b,a,r diff kwarg
4) Alternatively save your macros in a separate
   file, e.g. "mymacros.html" and load it to the
   current template with:
        {% loadkwacros "mymacros.html" %}
   Then use these loaded macros in {% usekwacro %}
   as described above.
Bear in mind that defined and loaded kwacros are local
to each template file and are not inherited
through {% extends ... %} tags.
"""

from django import template
from django.template.base import FilterExpression
from django.template.loader import get_template

register = template.Library()


def _setup_macros_dict(parser):
    ## Metadata of each macro are stored in a new attribute
    ## of 'parser' class. That way we can access it later
    ## in the template when processing 'usemacro' tags.
    try:
        ## Only try to access it to eventually trigger an exception
        parser._macros
    except AttributeError:
        parser._macros = {}


class DefineMacroNode(template.Node):
    def __init__(self, name, nodelist, args):

        self.name = name
        self.nodelist = nodelist
        self.args = []
        self.kwargs = {}
        for a in args:
            if "=" not in a:
                self.args.append(a)
            else:
                name, value = a.split("=")
                self.kwargs[name] = value

    def render(self, context):
        ## empty string - {% macro %} tag does no output
        return ''


@register.tag(name="kwacro")
def do_macro(parser, token):
    try:
        args = token.split_contents()
        tag_name, macro_name, args = args[0], args[1], args[2:]
    except IndexError:
        m = ("'%s' tag requires at least one argument (macro name)"
            % token.contents.split()[0])
        raise template.TemplateSyntaxError(m)
    # TODO: could do some validations here,
    # for now, "blow your head clean off"
    nodelist = parser.parse(('endkwacro', ))
    parser.delete_first_token()

    ## Metadata of each macro are stored in a new attribute
    ## of 'parser' class. That way we can access it later
    ## in the template when processing 'usemacro' tags.
    _setup_macros_dict(parser)
    parser._macros[macro_name] = DefineMacroNode(macro_name, nodelist, args)
    return parser._macros[macro_name]


class LoadMacrosNode(template.Node):
    def render(self, context):
        ## empty string - {% loadmacros %} tag does no output
        return ''


@register.tag(name="loadkwacros")
def do_loadmacros(parser, token):
    try:
        tag_name, filename = token.split_contents()
    except IndexError:
        m = ("'%s' tag requires at least one argument (macro name)"
            % token.contents.split()[0])
        raise template.TemplateSyntaxError(m)
    if filename[0] in ('"', "'") and filename[-1] == filename[0]:
        filename = filename[1:-1]
    t = get_template(filename)
    macros = t.nodelist.get_nodes_by_type(DefineMacroNode)
    ## Metadata of each macro are stored in a new attribute
    ## of 'parser' class. That way we can access it later
    ## in the template when processing 'usemacro' tags.
    _setup_macros_dict(parser)
    for macro in macros:
        parser._macros[macro.name] = macro
    return LoadMacrosNode()


class UseMacroNode(template.Node):

    def __init__(self, macro, fe_args, fe_kwargs):
        self.macro = macro
        self.fe_args = fe_args
        self.fe_kwargs = fe_kwargs

    def render(self, context):

        for i, arg in enumerate(self.macro.args):
            try:
                fe = self.fe_args[i]
                context[arg] = fe.resolve(context)
            except IndexError:
                context[arg] = ""

        for name, default in iter(self.macro.kwargs.items()):
            if name in self.fe_kwargs:
                context[name] = self.fe_kwargs[name].resolve(context)
            else:
                context[name] = FilterExpression(default,
                                                 self.macro.parser
                ).resolve(context)

        return self.macro.nodelist.render(context)


@register.tag(name="usekwacro")
def do_usemacro(parser, token):
    try:
        args = token.split_contents()
        tag_name, macro_name, values = args[0], args[1], args[2:]
    except IndexError:
        m = ("'%s' tag requires at least one argument (macro name)"
             % token.contents.split()[0])
        raise template.TemplateSyntaxError(m)
    try:
        macro = parser._macros[macro_name]
    except (AttributeError, KeyError):
        m = "Macro '%s' is not defined" % macro_name
        raise template.TemplateSyntaxError(m)

    fe_kwargs = {}
    fe_args = []

    for val in values:
        if "=" in val:
            # kwarg
            name, value = val.split("=")
            fe_kwargs[name] = FilterExpression(value, parser)
        else:  # arg
            # no validation, go for it ...
            fe_args.append(FilterExpression(val, parser))

    macro.parser = parser
    return UseMacroNode(macro, fe_args, fe_kwargs)



import pytest
from pytest_data import use_data

from ..utils import get_user_by_uidb64, pk_to_uidb64, uidb64_to_pk

UIDB64_TEST_DATA = (
    'pk, uid',
    [
        ('1', 'MQ'),
        ('99', 'OTk'),
        ('100', 'MTAw'),
        ('123456', 'MTIzNDU2')
    ]
)


@pytest.mark.parametrize(*UIDB64_TEST_DATA)
def test_uidb64_to_pk(pk, uid):
    assert uidb64_to_pk(uid) == pk


@pytest.mark.parametrize(*UIDB64_TEST_DATA)
def test_pk_to_uidb64(pk, uid):
    assert pk_to_uidb64(pk) == uid.encode()


@use_data(user_data={'id': 1})
def test_get_user_by_uidb64(user):
    assert user.pk == 1
    assert user == get_user_by_uidb64("MQ")


@use_data(user_data={'id': 1})
def test_get_user_by_uidb64_user_not_exits(user):
    user = get_user_by_uidb64("Mg")
    assert not user



from cms_qe_test import render_plugin

from ..cms_plugins import NewsletterPlugin


def test_render():
    html = render_plugin(NewsletterPlugin)
    assert '<form' in html
    assert '<input' in html
    assert 'email' in html
    assert 'first' not in html
    assert 'last' not in html
    assert 'submit' in html


def test_render_fullname_show():
    html = render_plugin(NewsletterPlugin, fullname_show=True)
    assert 'email' in html
    assert 'first' in html
    assert 'last' in html
    assert 'submit' in html



from django.conf.urls import url
from django.contrib.auth import views as django_views

from cms_qe_auth.forms import PasswordResetFormWithEmailExistenceCheck
from . import views

urlpatterns = [
    url(
        r'^login/$',
        django_views.login,
        {'template_name': 'cms_qe/auth/login.html'},
        name='login',
    ),
    url(
        r'^logout/$',
        django_views.logout,
        {'template_name': 'cms_qe/auth/logged_out.html'},
        name='logout',
    ),
    url(
        r'^password_change/$',
        django_views.password_change,
        {'template_name': 'cms_qe/auth/password_change_form.html'},
        name='password_change',
    ),
    url(
        r'^password_change/done/$',
        django_views.password_change_done,
        {'template_name': 'cms_qe/auth/password_change_done.html'},
        name='password_change_done',
    ),
    url(
        r'^password_reset/$',
        django_views.password_reset,
        {
            'template_name': 'cms_qe/auth/password_reset_form.html',
            'email_template_name': 'cms_qe/auth/password_reset_email.txt',
            'html_email_template_name': 'cms_qe/auth/password_reset_email.html',
            'password_reset_form': PasswordResetFormWithEmailExistenceCheck,
        },
        name='password_reset',
    ),
    url(
        r'^password_reset/done/$',
        django_views.password_reset_done,
        {'template_name': 'cms_qe/auth/password_reset_done.html'},
        name='password_reset_done',
    ),
    url(
        r'^reset/(?P<uidb64>[0-9A-Za-z_\-]+)/(?P<token>[0-9A-Za-z]{1,13}-[0-9A-Za-z]{1,20})/$',
        django_views.password_reset_confirm,
        {'template_name': 'cms_qe/auth/password_reset_confirm.html'},
        name='password_reset_confirm',
    ),
    url(
        r'^reset/done/$',
        django_views.password_reset_complete,
        {'template_name': 'cms_qe/auth/password_reset_complete.html'},
        name='password_reset_complete',
    ),
    url(
        r'^register/$',
        views.register,
        {'template_name': 'cms_qe/auth/register.html'},
        name='register'
    ),
    url(
        r'^activate/(?P<uidb64>[0-9A-Za-z_\-]+)/(?P<token>[0-9A-Za-z]{1,13}-[0-9A-Za-z]{1,20})/$',
        views.activate,
        {
            'template_name_complete': 'cms_qe/auth/email_confirmation_complete.html',
            'template_name_fail': 'cms_qe/auth/email_confirmation_fail.html',
        },
        name='activate'
    ),
]



from typing import Callable

from cms.views import details
from django.conf import settings
from django.http import HttpRequest, Http404
from django.http.response import HttpResponseBase
from django.shortcuts import render

__all__ = ('handler403', 'handler404', 'handler500', 'handler503', 'get_error_handler')


def get_error_handler(code: int) -> Callable[[HttpRequest], HttpResponseBase]:
    """
    Decorator creating error handler for specific HTTP status code.

    When CMS page with slug ``errorCODE`` exists, then is shown that
    page, otherwise generic one. Page is not cached because of that.

    Possible slugs:

     * error403 (forbidden)
     * error404 (not found)
     * error503 (service unavailable)
    """
    # pylint: disable=missing-docstring,unused-argument
    def error_handler(request: HttpRequest, *args, **kwds) -> HttpResponseBase:
        slug = 'error{}'.format(code)

        if hasattr(request, '_current_page_cache'):
            delattr(request, '_current_page_cache')

        try:
            response = details(request, slug)
        except Http404:
            response = render(request, 'cms_qe/error.html', context={'slug': slug})

        # In debug any response with status_code 404 is transformed to Django error
        # page and is not possible to check that page. We want to be able to check
        # not found page in debug as well.
        if response and not settings.DEBUG:
            response.status_code = code

        return response
    return error_handler


# pylint: disable=invalid-name
handler403 = get_error_handler(403)
handler404 = get_error_handler(404)
handler503 = get_error_handler(503)


def handler500(request: HttpRequest) -> HttpResponseBase:
    """
    When application fail on internal error, Django CMS has problem
    to render any page. They don't have any API for rendering any page
    so it's imposible to get it right. Solution could be just redirect
    to page with given slug but it could also hang in infinite loop of
    redirection if there would be problem on page with slug error500.

    That's why we simply just use different template which cannot be
    changed in admin by user. To override default template, just create
    your own ``cms_qe/internal_error.html`` or change ``handler500``
    to your own view.
    """
    return render(request, 'cms_qe/internal_error.html')



"""
URL Configuration
https://docs.djangoproject.com/en/1.11/topics/http/urls/
"""

from django.conf.urls import url

from .views import get_table_choices


urlpatterns = [
    url(r'^cms-qe/table/data', get_table_choices, name='get_table_choices'),
]
